<?php
require_once 'Phunctional/Iterator.php';
require_once dirname(__FILE__) . '/Statement.php';
require_once dirname(__FILE__) . '/Transaction.php';
require_once dirname(__FILE__) . '/Exception.php';

final class Phostgres_Connection {
	const Instance = 'pg_connect';
	const Persistance = 'pg_pconnect'; 

	protected static $connections = array(
		self::Instance => array(),
		self::Persistance => array()
	);

	public $resource, $databaseName, $statements = array();

	static function establish($connectionString, $mode = self::Instance) {
		if(!empty(self::$connections[$mode][$connectionString]))
			return self::$connections[$mode][$connectionString];
		return new self($connectionString, $mode);
	}

	function __construct($connectionString, $mode = self::Instance) {
		if(!$this->resource = @call_user_func($mode, $connectionString, true))
			$this->throwException('Phostgres_Connection_Exception');

		self::$connections[$mode][$connectionString] = $this;
		$this->databaseName = pg_dbname($this->resource);
	}

	function __destruct() {
		pg_close($this->resource);
	}

	function prepare($sql) {
		return new Phostgres_Statement($this, $sql);
	}

	function query($sql, $parameters = array()) {
		if(!is_array($parameters))
			$parameters = iterator_to_array(Iterator($parameters));

		$result = count($parameters)
				? @pg_query_params($this->resource, $sql, $parameters)
				: @pg_query($this->resource, $sql);

		if(!$result)
			$this->throwException('Phostgres_Statement_Exception');

		return Phostgres_ResultSet::fromResource($result);
	}

	function begin() {
		return new Phostgres_Transaction($this);
	}

	protected function throwException($class = 'Phostgres_Exception') {
		$err = error_get_last();
		$err = preg_replace('/^\s*pg_\w+\s*\(\s*\):\s*/', '', $err['message']);

		throw new $class($err);
	}
}

class Phostgres_Connection_Exception extends Phostgres_Exception {}
