<?php

class Phostgres_ResultSet implements Iterator, Countable {
	public $resource, $length;
	protected $currentRow, $currentKey = 0;

	protected function assertResourceType($res) {
		if(!is_resource($res) or get_resource_type($res) != 'pgsql result') {
			if('resource' == $type = gettype($res))
				$type = "'" . get_resource_type($res) . "' resource";
			else if($type == 'object')
				$type = 'instance of ' . get_class($type);

			throw new InvalidArgumentException(
				'Argument 1 passed to Phostgres_ResultSet::__construct()' .
				"must be an 'pgsql result' resource, $type given"
			);
		}
	}

	static function fromResource($resource) {
		self::assertResourceType($resource);

		switch(pg_result_status($resource)) {
			case PGSQL_TUPLES_OK:
				return new Phostgres_ResultSet($resource);
			case PGSQL_COMMAND_OK:
				return pg_affected_rows($resource);
		}
	}

	function __construct($resource) {
		self::assertResourceType($resource);
		$this->resource = $resource;
		$this->length = pg_num_rows($resource);
	}

	function __isset($name) {
		return $name == 'fields';
	}

	function __get($name) {
		if($name == 'fields') {
			for($i = 0, $n = pg_num_fields($this->resource); $i < $n; ++$i) {
				$this->fields[] = (object) array(
					'name' => pg_field_name($this->resource, $i),
					'type' => pg_field_type($this->resource, $i),
					'table' => pg_field_table($this->resource, $i)
				);

				if(!$this->fields[$i]->table)
					$this->fields[$i]->table = null;
			}

			return $this->fields;
		}
	}

	protected function fetch() {
		$this->currentRow = pg_fetch_array($this->resource);
	}

	function valid() {
		return $this->currentKey < $this->length;
	}

	function key() {
		return $this->currentKey;
	}

	function current() {
		if($this->currentKey < 1)
			$this->fetch();
		return $this->currentRow;
	}

	function next() {
		$this->fetch();
		++$this->currentKey;
	}

	function rewind() {
		$this->currentKey = 0;
		pg_result_seek($this->resource, 0);
	}

	function count() {
		return $this->length;
	}

	function toArray() {
		$array = array();
		while($row = pg_fetch_array($this->resource))
			$array[] = $row;
		pg_result_seek($this->resource, 0);
		return $array;
	}

	function __destruct() {
		pg_free_result($this->resource);
	}
}

