<?php
require_once 'Phunctional/Functor.php';
require_once 'Phunctional/Iterator.php';
require_once dirname(__FILE__) . '/Connection.php';
require_once dirname(__FILE__) . '/ResultSet.php';
require_once dirname(__FILE__) . '/Exception.php';

class Phostgres_Statement extends Functor {
	const AnonymousNamePattern = '__Phostgres_Statement__anonymous_%d';
	public $connection, $resource, $name;
	protected $error = null;

	function __construct(Phostgres_Connection $connection, $sql, $name = '') {
		settype($name, 'string');

		if($name == '') {
			do {
				isset($i) ? ++$i : $i = 1;
				$name = sprintf(self::AnonymousNamePattern, $i);
			}
			while(!empty($connection->statements[$name]));
		}
		else if(!empty($connection->statements[$name])) {
			throw new Phostgres_Statement_NameDuplicationException(
				"Prepared statement '$name' already exists"
			);
		}

		$this->name = $name;
		$this->connection = $connection;
		set_error_handler(array($this, 'setError'));
		$this->resource = pg_prepare($connection->resource, $this->name, $sql);
		restore_error_handler();

		if($this->error)
			throw new Phostgres_Statement_Exception($this->error);

		$connection->statements[$name] = $this;
	}

	protected function setError($_, $message) {
		$this->error = $message;
	}

	function execute($params = array()) {
		if(!is_array($params))
			$parames = iterator_to_array(Iterator($params));

		$result = pg_execute($this->connection->resource, $this->name, $params);
		return Phostgres_ResultSet::fromResource($result);
	}

	function apply(array $parameters) {
		return $this->execute($parameters);
	}
}

class Phostgres_Statement_Exception extends Phostgres_Exception {}
class Phostgres_Statement_NameDuplicationException
	extends Phostgres_Statement_Exception {}
