<?php
require_once dirname(__FILE__) . '/TestCase.php';
require_once dirname(__FILE__) . '/../Phostgres/Connection.php';

final class ConnectionTest extends Phostgres_TestCase {
	function testEstablish() {
		$this->assertType('Phostgres_Connection', $this->connection);
		$this->assertType('resource', $this->connection->resource);

		$this->assertEquals(
			'pgsql link',
			get_resource_type($this->connection->resource)
		);

		$this->assertEquals(
			PGSQL_CONNECTION_OK,
			pg_connection_status($this->connection->resource)
		);
	}

	function testWrongConnection() {
		try {
			Phostgres_Connection::establish('dbname=wrong_db_name_to_test');
			$this->fail();
		}
		catch(Phostgres_Connection_Exception $e) {
			# okay.
		}
	}

	function testDatabaseName() {
		preg_match('/dbname=(\w+)/', $this->connectionString, $matches);
		$this->assertEquals($matches[1], $this->connection->databaseName);
	}

	function testPrepare() {
		$stmt = $this->connection->prepare('SELECT 1');
		$this->assertType('Phostgres_Statement', $stmt);
	}

	function testPrepareWrongly() {
		try {
			$this->connection->prepare('WRONG SQL');
			$this->fail();
		}
		catch(Phostgres_Statement_Exception $e) {
			# okey.
		}
	}

	function testQuery() {
		$result = $this->connection->query('SELECT 1 AS a');
		$this->assertType('Phostgres_ResultSet', $result);
		$this->assertEquals(array(array(1, 'a' => 1)), $result->toArray());
	}

	function testQueryWithParameters() {
		$result = $this->connection->query(
			'VALUES ($1::int, $2::text), ($3::int, $4::text)',
			array(1, 'one', 2, 'two')
		);

		$this->assertType('Phostgres_ResultSet', $result);

		$this->assertEquals(
			array(
				array(1, 'one', 'column1' => 1, 'column2' => 'one'),
				array(2, 'two', 'column1' => 2, 'column2' => 'two'),
			),
			$result->toArray()
		);
	}

	function testQueryWithNullParameter() {
		$result = $this->connection->query('SELECT $1::int', array(null));
		$this->assertNull(current($result->current()));
	}

	function testResultOfNonSelectQuery() {
		$result = $this->connection->query(
			'CREATE TEMP SEQUENCE phostgres_query_test_seq'
		);

		$this->assertEquals(0, $result);
	}

	function testQueryWrongly() {
		try {
			$this->connection->query('WRONG SQL');
			$this->fail();
		}
		catch(Phostgres_Statement_Exception $e) {
			# okay.
		}
	}

	function testBegin() {
		$transaction = $this->connection->begin();
		$this->assertType('Phostgres_Transaction', $transaction);
	}
}
