<?php
require_once dirname(__FILE__) . '/TestCase.php';
require_once dirname(__FILE__) . '/../Phostgres/ResultSet.php';

final class ResultSetTest extends Phostgres_TestCase {
	static $expected = array(
		array(1, 'first', 'id' => 1, 'value' => 'first'),
		array(2, 'second', 'id' => 2, 'value' => 'second'),
		array(3, 'third', 'id' => 3, 'value' => 'third'),
		array(4, 'fourth', 'id' => 4, 'value' => 'fourth'),
		array(5, 'fifth', 'id' => 5, 'value' => 'fifth'),
		array(6, 'sixth', 'id' => 6, 'value' => 'sixth')
	);

	public $result;

	function setUp() {
		parent::setUp();

		$stmt = $this->connection->prepare("
			SELECT column1 AS id, column2 AS value FROM (
				VALUES (1, 'first'), (2, 'second'), (3, 'third'),
					(4, 'fourth'), (5, 'fifth'), (6, 'sixth')
			) AS a
		");

		$this->result = $stmt->execute();
	}

	function testFromResource() {
		$selection = pg_query($this->connection->resource, 'VALUES(1,2),(3,4)');
		$result = Phostgres_ResultSet::fromResource($selection);
		$this->assertType('Phostgres_ResultSet', $result);

		$creation = pg_query(
			$this->connection->resource,
			'CREATE TEMP SEQUENCE phostgres_resultset_test_seq'
		);

		$result = Phostgres_ResultSet::fromResource($creation);
		$this->assertEquals(0, $result);
	}

	function testIterator() {
		$this->assertType('Traversable', $this->result);
	}

	function testSequence() {
		$this->assertEquals(self::$expected, iterator_to_array($this->result));
	}

	function testRewind() {
		for(; $this->result->valid(); $this->result->next());
		$this->result->rewind();

		$this->assertEquals(0, $this->result->key());

		$this->assertEquals(
			array(1, 'first', 'id' => 1, 'value' => 'first'),
			$this->result->current()
		);

		$this->result->next();

		$this->assertEquals(1, $this->result->key());

		$this->assertEquals(
			array(2, 'second', 'id' => 2, 'value' => 'second'),
			$this->result->current()
		);
	}

	function testNullValue() {
		$result = $this->connection->query('SELECT NULL');
		$this->assertNull(current($result->current()));
	}

	function testToArray() {
		$this->assertEquals(self::$expected, $this->result->toArray());
		$this->assertEquals(self::$expected, $this->result->toArray());
	}

	function testCount() {
		$this->assertEquals(6, count($this->result));
		$this->assertEquals(6, $this->result->length);
	}

	function testFields() {
		$this->assertTrue(isset($this->result->fields));
		$this->assertEquals(2, count($this->result->fields));
	}

	function testFieldNames() {
		$this->assertEquals('id', $this->result->fields[0]->name);
		$this->assertEquals('value', $this->result->fields[1]->name);
	}

	function testFieldTypes() {
		$this->assertEquals('int4', $this->result->fields[0]->type);
		$this->assertEquals('text', $this->result->fields[1]->type);
	}

	function testFieldTables() {
		$this->assertNull($this->result->fields[0]->table);
		$this->assertNull($this->result->fields[1]->table);
	}
}
