<?php
require_once dirname(__FILE__) . '/TestCase.php';
require_once dirname(__FILE__) . '/../Phostgres/Statement.php';

final class StatementTest extends Phostgres_TestCase {
	function testFunctor() {
		$this->assertType(
			'Callable',
			new Phostgres_Statement($this->connection, 'SELECT 1')
		);
	}

	function testExecute() {
		$stmt = new Phostgres_Statement(
			$this->connection, '
			SELECT column1 AS id, column2 AS name FROM (
				VALUES (1, $1::varchar), (2, $2::varchar)
			) a
		');

		$expected = array(
			array(1, 'Hong, MinHee', 'id' => 1, 'name' => 'Hong, MinHee'),
			array(2, null, 'id' => 2, 'name' => null)
		);

		$result = $stmt->execute(array('Hong, MinHee', null));

		$this->assertType('Phostgres_ResultSet', $result);
		$this->assertEquals($expected, $result->toArray());

		$this->assertEquals(
			$expected,
			$stmt->apply(array('Hong, MinHee', null))->toArray()
		);

		$this->assertEquals(
			$expected,
			$stmt->call('Hong, MinHee', null)->toArray()
		);
	}

	function testName() {
		$stmt = new Phostgres_Statement($this->connection, 'SELECT 1', 'test');

		$this->assertEquals('test', $stmt->name);
		$this->assertSame($stmt, $this->connection->statements['test']);
	}

	function testTryOverwriteName() {
		new Phostgres_Statement($this->connection, 'SELECT 1', 'test2');

		try {
			new Phostgres_Statement($this->connection, 'SELECT 2', 'test2');
			$this->fail();
		}
		catch(Phostgres_Statement_NameDuplicationException $e) {
			# okay.
		}
	}

	function testImplicitName() {
		$stmt = new Phostgres_Statement($this->connection, 'SELECT 1');

		$this->assertRegExp(
			'/__Phostgres_Statement__anonymous_\d+/',
			$stmt->name
		);
	}

	function testResultOfNonSelectQuery() {
		$stmt = $this->connection->prepare(
			'CREATE TEMP SEQUENCE phostgres_stmt_test_seq'
		);

		$this->assertEquals(0, $stmt->execute());
	}

	function testNullParameter() {
		$stmt = $this->connection->prepare('SELECT $1::int');
		$this->assertNull(current($stmt->execute(array(null))->current()));
	}
}
