<?php
require_once dirname(__FILE__) . '/TestCase.php';
require_once dirname(__FILE__) . '/../Phostgres/Transaction.php';

final class TransactionTest extends Phostgres_TestCase {
	function testBegin() {
		$transaction = $this->connection->begin();
		$this->assertType('Phostgres_Transaction', $transaction);
		$this->assertSame($this->connection, $transaction->connection);
		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);
	}

	function testConstruct() {
		$transaction = new Phostgres_Transaction($this->connection);
		$this->assertType('Phostgres_Transaction', $transaction);
		$this->assertSame($this->connection, $transaction->connection);
		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);
	}

	function testRollback() {
		$table = 'phostgres_transaction_test';
		$this->connection->query("DROP TABLE IF EXISTS $table");
		$this->connection->query("CREATE TEMP TABLE $table ( v int )");
		$this->connection->query("INSERT INTO $table VALUES (1)");
		$transaction = $this->connection->begin();
		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);

		$result = $this->connection->query("SELECT count(v) FROM $table");
		$this->assertEquals(1, current($result->current()));

		$this->connection->query(
			"INSERT INTO $table (v) VALUES ($1)",
			array(123)
		);

		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);

		$result = $this->connection->query("SELECT count(v) FROM $table");
		$this->assertEquals(2, current($result->current()));

		$transaction->rollback();

		$result = $this->connection->query("SELECT count(v) FROM $table");
		$this->assertEquals(1, current($result->current()));

		$this->assertEquals(
			Phostgres_Transaction::NonTransaction,
			$transaction->status
		);
	}

	function testCommit() {
		$table = 'phostgres_transaction_test';
		$this->connection->query("DROP TABLE IF EXISTS $table");
		$this->connection->query("CREATE TEMP TABLE $table ( v int )");

		$result = $this->connection->query("SELECT count(v) FROM $table");
		$this->assertEquals(0, current($result->current()));

		$transaction = $this->connection->begin();
		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);
		$this->connection->query("INSERT INTO $table VALUES (1)");
		$this->assertEquals(Phostgres_Transaction::Idle, $transaction->status);
		$transaction->commit();

		$result = $this->connection->query("SELECT count(v) FROM $table");
		$this->assertEquals(1, current($result->current()));

		$this->assertEquals(
			Phostgres_Transaction::NonTransaction,
			$transaction->status
		);

		$this->connection->query("DROP TABLE IF EXISTS $table");
	}

	function testQuery() {
		$transaction = $this->connection->begin();
		$result = $transaction->query('SELECT 1 AS a');
		$this->assertType('Phostgres_ResultSet', $result);
		$this->assertEquals(array(array(1, 'a' => 1)), $result->toArray());
	}

	function testQueryWithParameters() {
		$transaction = $this->connection->begin();
		$result = $transaction->query(
			'VALUES ($1::int, $2::text), ($3::int, $4::text)',
			array(1, 'one', 2, 'two')
		);

		$this->assertType('Phostgres_ResultSet', $result);

		$this->assertEquals(
			array(
				array(1, 'one', 'column1' => 1, 'column2' => 'one'),
				array(2, 'two', 'column1' => 2, 'column2' => 'two'),
			),
			$result->toArray()
		);
	}
}
